@@ -26,6 +26,8 @@ import time
 from collections import namedtuple
 from configparser import ConfigParser
 from hashlib import sha256, md5
+from signal import signal, SIGINT, SIGTERM, SIGABRT
+from threading import Event
 
 from pyrogram.api import functions, types
 from pyrogram.api.core import Object
@@ -71,6 +73,7 @@ class Client:
         self.session = None
 
         self.update_handler = None
+        self.is_idle = Event()
 
     # TODO: Better update handler
     def set_update_handler(self, callback: callable):
@@ -79,6 +82,16 @@ class Client:
     def send(self, data: Object):
         return self.session.send(data)
 
+    def signal_handler(self, *args):
+        self.stop()
+        self.is_idle.set()
+
+    def idle(self, stop_signals: tuple = (SIGINT, SIGTERM, SIGABRT)):
+        for s in stop_signals:
+            signal(s, self.signal_handler)
+
+        self.is_idle.wait()
+
     def authorize(self):
         while True:
             phone_number = input("Enter phone number: ")
@@ -326,7 +339,7 @@ class Client:
         )
 
         offset_date = parse_dialogs(dialogs)
-        logging.info("Dialogs count: {}".format(len(peers)))
+        log.info("Dialogs count: {}".format(len(peers)))
 
         while len(dialogs.dialogs) == self.DIALOGS_AT_ONCE:
             dialogs = self.send(
@@ -337,7 +350,7 @@ class Client:
             )
 
             offset_date = parse_dialogs(dialogs)
-            logging.info("Dialogs count: {}".format(len(peers)))
+            log.info("Dialogs count: {}".format(len(peers)))
 
         for i in peers:
             peer_id = i["id"]
@@ -580,7 +593,7 @@ class Client:
             functions.messages.SendMedia(
                 peer=self.resolve_peer(chat_id),
                 media=types.InputMediaUploadedDocument(
-                    mime_type=mimetypes.types_map["." + audio.split(".")[-1]],
+                    mime_type=mimetypes.types_map.get("." + audio.split(".")[-1], "audio/mpeg"),
                     file=self.save_file(audio),
                     caption=caption,
                     attributes=[
@@ -620,3 +633,118 @@ class Client:
                 random_id=self.rnd_id()
             )
         )
+
+    def send_video(self,
+                   chat_id: int or str,
+                   video: str,
+                   duration: int = 0,
+                   width: int = 0,
+                   height: int = 0,
+                   caption: str = "",
+                   disable_notification: bool = None,
+                   reply_to_message_id: int = None):
+        return self.send(
+            functions.messages.SendMedia(
+                peer=self.resolve_peer(chat_id),
+                media=types.InputMediaUploadedDocument(
+                    mime_type=mimetypes.types_map[".mp4"],
+                    file=self.save_file(video),
+                    caption=caption,
+                    attributes=[
+                        types.DocumentAttributeVideo(
+                            duration=duration,
+                            w=width,
+                            h=height
+                        )
+                    ]
+                ),
+                silent=disable_notification or None,
+                reply_to_msg_id=reply_to_message_id,
+                random_id=self.rnd_id()
+            )
+        )
+
+    def send_voice(self,
+                   chat_id: int or str,
+                   voice: str,
+                   caption: str = "",
+                   duration: int = 0,
+                   disable_notification: bool = None,
+                   reply_to_message_id: int = None):
+        return self.send(
+            functions.messages.SendMedia(
+                peer=self.resolve_peer(chat_id),
+                media=types.InputMediaUploadedDocument(
+                    mime_type=mimetypes.types_map.get("." + voice.split(".")[-1], "audio/mpeg"),
+                    file=self.save_file(voice),
+                    caption=caption,
+                    attributes=[
+                        types.DocumentAttributeAudio(
+                            voice=True,
+                            duration=duration
+                        )
+                    ]
+                ),
+                silent=disable_notification or None,
+                reply_to_msg_id=reply_to_message_id,
+                random_id=self.rnd_id()
+            )
+        )
+
+    def send_video_note(self,
+                        chat_id: int or str,
+                        video_note: str,
+                        duration: int = 0,
+                        length: int = 1,
+                        disable_notification: bool = None,
+                        reply_to_message_id: int = None):
+        return self.send(
+            functions.messages.SendMedia(
+                peer=self.resolve_peer(chat_id),
+                media=types.InputMediaUploadedDocument(
+                    mime_type=mimetypes.types_map[".mp4"],
+                    file=self.save_file(video_note),
+                    caption="",
+                    attributes=[
+                        types.DocumentAttributeVideo(
+                            round_message=True,
+                            duration=duration,
+                            w=length,
+                            h=length
+                        )
+                    ]
+                ),
+                silent=disable_notification or None,
+                reply_to_msg_id=reply_to_message_id,
+                random_id=self.rnd_id()
+            )
+        )
+
+    def send_venue(self,
+                   chat_id: int or str,
+                   latitude: float,
+                   longitude: float,
+                   title: str,
+                   address: str,
+                   foursquare_id: str = "",
+                   disable_notification: bool = None,
+                   reply_to_message_id: int = None):
+        return self.send(
+            functions.messages.SendMedia(
+                peer=self.resolve_peer(chat_id),
+                media=types.InputMediaVenue(
+                    geo_point=types.InputGeoPoint(
+                        lat=latitude,
+                        long=longitude
+                    ),
+                    title=title,
+                    address=address,
+                    provider="",
+                    venue_id=foursquare_id,
+                    venue_type=""
+                ),
+                silent=disable_notification or None,
+                reply_to_msg_id=reply_to_message_id,
+                random_id=self.rnd_id()
+            )
+        )
